@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2016 ProcessOne, SARL. All Rights Reserved.
+ * Copyright (C) 2002-2017 ProcessOne, SARL. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -91,12 +91,12 @@ static attrs_list_t stream_stream_ns_attr = {
   FAKE_BIN("http://etherx.jabber.org/streams")
 };
 
-static char *jabber_client_ns = "jabber:client";
-
 static int same_str_buf(const char *str, const char *buf, size_t buf_len)
 {
   if (strlen(str) != buf_len)
     return 0;
+  if (!buf_len)
+    return 1;
   return memcmp(str, buf, buf_len) == 0;
 }
 
@@ -106,17 +106,13 @@ static char *dup_buf(const char *buf, size_t buf_len)
   if (!res)
     return NULL;
 
-  memcpy(res, buf, buf_len);
+  if (buf_len)
+    memcpy(res, buf, buf_len);
   res[buf_len] = '\0';
 
   return res;
 }
 
-static char *dup_str(const char *str)
-{
-  return dup_buf(str, strlen(str));
-}
-
 static int dup_to_bin(ErlNifBinary *bin, const char *buf, size_t buf_len)
 {
   if (!enif_alloc_binary(buf_len, bin))
